#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "st7565r.h"
#include "spi.h"

enum state {OFF, ON};
enum boolean {NO, YES};

// column address <-> segment driver direction (ADC Select)
#define SEG_REV(x) 0b10100000 | x
// page adress <-> common output direction
#define COM_REV(x) 0b11000000 | (x<<3)

// control pin commands:
#define SNDDT    A0_PORT |= (1<<A0)
#define SNDCMD   A0_PORT &= ~(1<<A0)
#define LCDSEL   LCD_CS_PORT &= ~(1<<LCD_CS)
#define LCDSYNC  LCD_CS_PORT |= (1<<LCD_CS)

#define LCD(x) 0b10101110 | x     // turn on/off the display
#define LCD_REV(x) 0b10100110 | x // reverse the display data
#define ALL_PTS(x) 0b10100100 | x // turn on all points
#define BIAS1_9 0b10100010 // 1/9 bias
#define BIAS1_7 0b10100011 // 1/7 bias
#define RESET 0b11100010
#define LCD_NOP 0b11100011

// power control set:
#define BST 2 // booster
#define REG 1 // regulator
#define FLW 0 // follower
#define POW_CTRL 0b00101000 | (1<<BST) | (1<<REG) | (1<<FLW) // single supply mode

// booster
#define BSTCMD        0b11111000
#define BSTSET_2x3x4x 0b00000000
#define BSTSET_5x     0b00000001
#define BSTSET_6x     0b00000011

#define VREG 0b00100000 | vreg // 5 MSBs and 3 LSBs

#define VOLCMD 0b10000001
#define VOLVAL 0b00111111 & vol

// static indicator (double byte command):
#define STIND(x)  0b10101100 | x    // ON/OFF
#define STIND_SET(x) 0b00000011 & x // 0=OFF, 1=ON(1Hz blinking), 2=ON(2Hz blinking), 3=ON(const. on)

#define SLINE 0b01000000 | sline // 2 MSBs and 6 LSBs
#define PADDR 0b10110000 | paddr // 4 MSBs and 4 LSBs
#define CADDR_MSB 0b00010000 | (caddr >> 4) // transmitted first
#define CADDR_LSB 0b00001111 & caddr

uint8_t vreg;  // 3 bit, V0 voltage regulator resistor ratio 0~7
uint8_t vol;   // 6 bit, volume 0~63
uint8_t sline; // 6 bit, start line 0 ~ 63
uint8_t caddr; // 8 bit, column address 0~127
uint8_t paddr; // 3 bit, page address 0 ~ 8

uint8_t lcd_data [1024];

void dogl_init (void) {
  spi_init();
  A0_DDR |= (1<<A0);
  LCD_CS_DDR |= (1<<LCD_CS);
  LCDSYNC;
  LCDSEL;
  SNDCMD;
  sline = 0;
  spi_transceive(SLINE);
  spi_transceive(SEG_REV(NO));
  spi_transceive(COM_REV(YES));
  spi_transceive(LCD_REV(NO));
  spi_transceive(BIAS1_9);
  spi_transceive(POW_CTRL);
  spi_transceive(BSTCMD);
  spi_transceive(BSTSET_2x3x4x);
  vreg = 7;
  spi_transceive(VREG);
  vol = 20;
  spi_transceive(VOLCMD);
  spi_transceive(VOLVAL);
  spi_transceive(STIND(OFF));
  spi_transceive(STIND_SET(OFF));
  spi_transceive(LCD(ON));
  spi_transceive(ALL_PTS(ON));
  _delay_ms(500);
  spi_transceive(ALL_PTS(OFF));
  LCDSYNC;
}

void menu_screen_set (unsigned char x, unsigned char y, unsigned char color) {
  lcd_data [128 * (y/8) + x + 4] = ((!color) << (y%8));
}
#define OFFSET 4
void menu_screen_flush (void) {
  LCDSEL;
  SNDCMD;
  sline = 0;
  spi_transceive(SLINE);
  for (paddr = 0; paddr < 8; paddr++) {
    SNDCMD;
    spi_transceive (PADDR);
    for (caddr = 0 + OFFSET; caddr < (128 + OFFSET); caddr++) {
      SNDCMD;
      spi_transceive (CADDR_MSB);
      spi_transceive (CADDR_LSB);
      SNDDT;
      spi_transceive ( lcd_data [128 * paddr + caddr] );
    }
  }
  LCDSYNC;
}

void menu_screen_clear (void) {
  LCDSEL;
  SNDCMD;
  sline = 0;
  spi_transceive(SLINE);
  for (paddr = 0; paddr < 8; paddr++) {
    SNDCMD;
    spi_transceive (PADDR);
    for (caddr = 0 + OFFSET; caddr < (128 + OFFSET); caddr++) {
      SNDCMD;
      spi_transceive (CADDR_MSB);
      spi_transceive (CADDR_LSB);
      SNDDT;
      spi_transceive (0x00);
    }
  }
  LCDSYNC;
}

void pgmspace_to_lcd (uint8_t * bitmap) {
  LCDSEL;
  SNDCMD;
  sline = 0;
  spi_transceive(SLINE);
  for (paddr = 0; paddr < 8; paddr++) {
    SNDCMD;
    spi_transceive (PADDR);
    for (caddr = 0 + OFFSET; caddr < (128 + OFFSET); caddr++) {
      SNDCMD;
      spi_transceive (CADDR_MSB);
      spi_transceive (CADDR_LSB);
      SNDDT;
      spi_transceive ( bitmap [128 * paddr + caddr] );
    }
  }
  LCDSYNC;
}
